@@ -30,20 +30,24 @@ def format_ip(ip: str) -> str:
         return '[{}]'.format(ip)
     return ip
 
+
 def url(ip, port, path):
     url = 'http://{}:{}'.format(format_ip(ip), port)
     return urllib.parse.urljoin(url, path)
 
+
 def parse_header_ip(s: str) -> str:
     return s.strip().rsplit(':', 1)[0].strip('[]')
 
+
 class ClusterService:
 
     def __init__(self, credentials):
         self.log = getLogger('pequod.cluster-service')
         self.credentials = credentials
         self.user_agent = 'pequod-cli/{} {}/{} {}/{}'.format(pequod_cli.__version__, platform.python_implementation(),
-                                                             platform.python_version(), platform.system(), platform.release())
+                                                             platform.python_version(), platform.system(),
+                                                             platform.release())
 
     @property
     def endpoints(self) -> list:
@@ -109,7 +113,8 @@ class ClusterService:
                 # do not log as we are a command line interface
                 # self.log.warning('Client Error: %s', response.text)
                 raise requests.HTTPError(http_error_msg, response=response)
-        raise requests.ConnectionError('Request to {} failed for all {} endpoint(s): {}'.format(self.name, len(endpoints), last_error))
+        raise requests.ConnectionError('Request to {} failed for all {} endpoint(s): {}'.format(
+            self.name, len(endpoints), last_error))
 
     def _prepare_data(self, kwargs: dict):
         """
@@ -132,7 +137,6 @@ class ClusterService:
         if self.credentials:
             kwargs['auth'] = (self.credentials['username'], self.credentials['password'])
 
-
     def delete(self, path, *args, **kwargs):
         return self._request('DELETE', requests.delete, path, *args, **kwargs)
 
@@ -173,11 +177,14 @@ class CoreService(ClusterService):
     def name(self):
         return self.service_name
 
-    def _request(self, method_name, method, path, use_credentials=True, *args, **kwargs):
+    def _request(self, method_name, method, path, use_credentials=True, recursion_depth=0, *args, **kwargs):
         try:
             return super()._request(method_name, method, path, use_credentials, *args, **kwargs)
         except requests.HTTPError as e:
             if e.response.status_code == 451:
+                if recursion_depth > len(self._rotated_endpoints):
+                    raise
+
                 # we are contacting a service which is not the master
                 master = e.response.headers.get('X-Pequod-Master')
                 if not master:
@@ -190,7 +197,8 @@ class CoreService(ClusterService):
                         break
                 self._rotated_endpoints = self._rotated_endpoints[1:] + self._rotated_endpoints[:1]
 
-                return self._request(method_name, method, path, use_credentials, *args, **kwargs)
+                return self._request(method_name, method, path, use_credentials,
+                                     recursion_depth=recursion_depth + 1, *args, **kwargs)
             raise
 
     def status(self):
